﻿package {
// SpringyLetters.as - Actionscript 3 Version
//
// Jim Bumgardner
//
// This is a higher-level class that provides the 'colored letters on a string'
// animation.  It uses the lowerlevel 'SpringSim' to accomplish the hard stuff.
//
// This class is mainly concerned with setting up the simulation and the initial arrangement
// of the letters, all of which happens in 'setupTitle' below.  Most of the interesting physics-related
// stuff happens in 'SpringSim', my lower-level spring-physics simulation.
//
// This object assumes the existence of a movie in the library called 'letter_mc' which
// is used to contruct letter tiles with.  It must contain a dynamic text field called 'letter_txt'
// with an embedded font.  The font won't rotate properly if the letters are not embedded.
//
	import flash.events.Event;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;

	public class SpringyLetters
	{
		public static var scaleToFit:Boolean = true;
		public static var rotateToFit:Boolean = true;
		public var anchor1, anchor2; // movieclips
		public var ss;
		var letlist = [];
		var springTautness,anchorX,anchorY,stringWidth,parMC;

		function SpringyLetters(parMC, gravK, stiffK, frictionK, springTautness, 
								initTitle, anchorX, anchorY, stringWidth)
		{
			this.parMC = parMC;
			this.ss = new SpringSim(parMC, gravK, stiffK, frictionK, true);
			this.springTautness = springTautness;
			this.anchorX = anchorX;	
			this.anchorY = anchorY;
			this.stringWidth = stringWidth;
			this.setupTitle(initTitle);
		}
		

		// This function can rotation and scale of each letter to match position on spring-path.
		//
		function letterAdjust(evt) // animation handler for letters
		{
			var mc = evt.target;
			var mc1 = letlist[mc.idx-1];   // neighboring movieclips on the string of letters
			var mc2 = letlist[mc.idx+1];
			var dx = mc2.x - mc1.x;			   // the distance between those clips
			var dy = mc2.y - mc1.y;
			if (rotateToFit)
				mc.rotation = Math.atan2(dy, dx)*180/Math.PI;
			else
				mc.rotation = 0;
			if (scaleToFit)
				mc.scaleX = mc.scaleY = (Math.sqrt(dx*dx+dy*dy)/2)/mc.subMC.orig_width;
			else
				mc.scaleX = mc.scaleY = 1;

		}

		// This is used to add anchor points for the string - each string begins and ends
		// with an invisible movieclip that is fixed into place.
		//
		
		function addAnchor(px,py):MassSprite
		{
			var mc = new MassSprite(ss);
			parMC.addChild(mc);
			mc.x = px;
			mc.y = py;
			mc.idx = letlist.length;
			mc.setDotProps(true,false);
			letlist.push(mc);
			// ss.registerDot(mc);
			return mc;
		}
		
		// This adds a colored letter to the string
		//
		function addLetter(px,py,letter,clr)
		{

			var mc = new letter_mc();
			mc.letter_txt.autoSize = TextFieldAutoSize.CENTER;
			mc.letter_txt.text = letter;
			mc.letter_txt.textColor = clr;
			mc.orig_width = mc.letter_txt.textWidth;
			
			var mcc = new MassSprite(ss, mc);
			mcc.setDotProps(false,false);
			mcc.x = px;
			mcc.y = py;
			parMC.addChild(mcc);
			mcc.idx = letlist.length;
			mcc.drawDot();
			mcc.addEventListener(Event.ENTER_FRAME, letterAdjust);
			letlist.push(mcc);
			// ss.registerDot(mcc);
			return mcc;
		}
		
		// Get a purty/ugly rainbow color for letters
		// For more info about this technique: http://www.krazydad.com/makecolors.php
		function getRainbowColor(r) // r is 0 - 1  - returns a bright color
		{
			var a = r*Math.PI*2;
			var r = 128+Math.cos(a)*127;
			var b = 128+Math.cos(a+2*Math.PI/3)*127;
			var g = 128+Math.cos(a+4*Math.PI/3)*127;
			return (r << 16) | (g << 8) | b;
		}
		
		// This does most of the set up work.
		// It creates a set of letters and sets up the springs
		//
		// For long titles or tight spaces, you may need to tweak this function
		// so that your strings work the way you want them to.
		//
		public function setupTitle(titleStr)
		{
			// Remove existing letters, if any
			trace("setting up text: " + titleStr);
			for (var i = 1; i < letlist.length-1; ++i)
			{
				letlist[i].removeEventListener(Event.ENTER_FRAME, letterAdjust);
			}
			letlist = [];
			this.ss.reset();
			
			// Figure out where the string goes and how many MCs we need
			var n = titleStr.length+2;
			var px = this.anchorX;
			var py = this.anchorY;
			var TW = (this.stringWidth-px*2)/n;
		
			// Setup our first anchor
			var mc = addAnchor(px, py);
			anchor1 = mc;
			px += 50;
			var lastMC = mc;

			// Loop thru the letters and add them, connecting everything with springs
			// Initially the letters are strung based on the width of each letter in the embedded font
		
			for (i = 0; i < titleStr.length; ++i) 
			{
				// we assign each letter a rainbow color here
				var clr = this.getRainbowColor(i/(.5+titleStr.length-1));
		
				mc = this.addLetter(px, py, titleStr.charAt(i), clr);
				// This adds the spring.  Normally when you add a spring between two points,
				// it is relaxed.  springTautness stretches it a bit by shortening the 'relaxed' length.
				// so it's more like a tight elastic.
				this.ss.addSpring(lastMC, mc, this.springTautness);
				px += mc.subMC.letter_txt.textWidth;
				lastMC = mc;
			}
		
			// Add the final anchor and spring
			px += 30;
			mc = this.addAnchor(px, py);
			anchor2 = mc;
			this.ss.addSpring(lastMC, mc, this.springTautness);
		
			// This stretches the final anchor to it's desired position, causing
			// the letters to careen joyfully due to the whole chain being stretched out.
			mc.x = this.anchorX + this.stringWidth;
			// trace(" nbr dots: " + this.ss.dots.length + ", " + this.ss.hasMC(mc));
			trace('letter setup done');
		}
	}
}
